/*
This programs tells you how to rotate images at a point other than origin.
click inisde to window to change the point of rotation

NOTE:
This tutorial tells you what is the method to rotate an image at some point.
This is not the cleanest/fastest way to do so.
You can simply call some SDK functions to do so; but its nice if you understand
what happens inside those functions.
      
*/

# include <math.h>
# include <stdio.h>
# include <stdlib.h>
# include <GL/glut.h>

# include "TgaTexture.h"

struct Point
{
    float x,y;
};

TgaTexture rocket;


float theta = 22.0f/14.0f;
int windowWidth = 800;
int windowHeight = 800;

//centerX and centerY represent the point
//in the screen around which the image rotates
//in other words, it is the center of rotation
//these values lie between -1 and +1
float centerX = 0.0f;
float centerY = 0.0f;


//When the user clicks the mouse,
//we change the image's location to the 
//point where the mouse was clicked
Point Translate(float x, float y)
{
    Point p;

    p.x = x + centerX;
    p.y = y + centerY;

    return p;
}


//When the user moves the mouse,
//we rotate the image so that
//the rocket image points towards the mouse
Point Rotate(float x, float y)
{
    Point p;

    p.x = x*cos(theta) - y*sin(theta);
    p.y = x*sin(theta) + y*cos(theta);

    return p;
}


void OnWindowResize(int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    windowWidth = width;
    windowHeight = height;
}


void OnKeyPress(unsigned char key, int x, int y)
{
# define ESC_Key 27

    switch (key)
    {
        case ESC_Key:
            exit(0);
        break;
    }
}


void Draw()
{
    Point p;
    float size = 0.125f;

    glClearColor(0.1f, 0.1f, 0.3f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT);


	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    rocket.Activate();

    glBegin(GL_TRIANGLE_STRIP);

    glTexCoord2f(0.0f, 0.0f);
    p = Rotate(-size, -size);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glTexCoord2f(1.0f, 0.0f);
    p = Rotate( size, -size);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);


    glTexCoord2f(0.0f, 1.0f);
    p = Rotate(-size,  size);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glTexCoord2f(1.0f, 1.0f);
    p = Rotate( size,  size);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glEnd();

    glutSwapBuffers();

}


void OnMouseMove(int mouseX, int mouseY)
{
    //Convert mouseX, mouseY into (x, y)
    //where x and y values are between -1 and +1
    float x = (float) (2.0f*mouseX)/(float)windowWidth - 1.0f;
    float y = 1.0f - (float)(2.0f*mouseY)/(float)windowHeight;

    //Now, calculate the x, y value, assuming that
    //origin is translated to (centerX, centerY)
    x -= centerX;
    y -= centerY;
    theta = atan2(y, x);

    glutPostRedisplay();
}


void OnMouseClick(int button, int state, int x, int y)
{
    //Convert mouseX, mouseY into (x, y)
    //where x and y values are between -1 and +1
    centerX = (float) (2.0f*x)/(float)windowWidth - 1.0f;
    centerY = 1.0f - (float)(2.0f*y)/(float)windowHeight;

    glutPostRedisplay();
}

int main(int argc, char** argv)
{
    const int left = 0;
    const int top = 0;

    const char* windowTitle = "Evolution of revolution - 2";

    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
    glutInitWindowSize(windowWidth, windowHeight); 
    glutInitWindowPosition(left, top);
    glutCreateWindow(windowTitle);
    
    glutDisplayFunc(Draw);
    glutReshapeFunc(OnWindowResize);
    glutKeyboardFunc(OnKeyPress);
    glutMouseFunc(OnMouseClick);
    glutIdleFunc(Draw);
    glutPassiveMotionFunc(OnMouseMove);

    if(!rocket.Init("rocket.tga")) return 1;


    glutMainLoop();

    return 0;
}