# include <math.h>
# include <stdio.h>
# include <stdlib.h>
# include <GL/glut.h>
# include <mmsystem.h> 

# include "Rocket.h"

static const float bombTime = 0.25f;
static const float rocketSize = 0.125f;

Rocket::Rocket() : x(0.0f), y(0.0f), theta(0.0f), timeLeft(0.0f)
{

}



void Rocket::Shoot()
{
	if(timeLeft <= 0.0f)
	{
		timeLeft = bombTime;
		PlaySound(L"shoot.wav", NULL, SND_ASYNC);
	}
}



Point Rocket::Translate(float x1, float y1)
{
    Point p;

    p.x = x + x1;
    p.y = y + y1;

    return p;
}



Point Rocket::Rotate(float x, float y)
{
    Point p;

    p.x = x*cos(theta) - y*sin(theta);
    p.y = x*sin(theta) + y*cos(theta);

    return p;
}

void Rocket::DrawBullet()
{
	glColor3f(1.0f, 1.0f, 0.0f);

	float magnitude = (bombTime - timeLeft)/bombTime;
	float x1 = x + cos(theta)*magnitude;
	float y1 = y + sin(theta)*magnitude;

	bullet.DrawQuad(x1, y1, 0.125f);


	glColor3f(1.0f, 1.0f, 1.0f);
}


void Rocket::Draw()
{
	if(timeLeft > 0.0f)
	{
		DrawBullet();

		shootingTexture.Activate();
	}
	else
	{
		idleTexture.Activate();
	}

    glBegin(GL_TRIANGLE_STRIP);

	Point p = {0.0f, 0.0f};

    glTexCoord2f(0.0f, 0.0f);
    p = Rotate(-rocketSize, -rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glTexCoord2f(1.0f, 0.0f);
    p = Rotate( rocketSize, -rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);


    glTexCoord2f(0.0f, 1.0f);
    p = Rotate(-rocketSize,  rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glTexCoord2f(1.0f, 1.0f);
    p = Rotate( rocketSize,  rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glEnd();
}




void Rocket::Init()
{
	idleTexture.Init("rocket.tga");
	shootingTexture.Init("rocket_firing.tga");
	bullet.Init("bullet.tga");
}



void Rocket::OnMouseMove(float x1, float y1)
{
	x1 -= x;
    y1 -= y;
    theta = atan2(y1, x1);
}


void Rocket::Update(float deltaTime)
{
	timeLeft -= deltaTime;
}