# include <math.h>
# include <stdio.h>
# include <stdlib.h>
# include <GL/glut.h>
# include <mmsystem.h> 

# include "Rocket.h"

static const float bombTime = 0.25f;
static const float rocketSize = 0.125f;

Rocket::Rocket() : x(0.5f), y(-0.5f), theta(0.0f), timeLeft(0.0f)
{

}



void Rocket::Shoot()
{
	if(timeLeft <= 0.0f)
	{
		timeLeft = bombTime;
	}
	
	PlaySound(L"shoot.wav", NULL, SND_ASYNC);
	missileManager.Add(x, y, theta);
}



Point Rocket::Translate(float x1, float y1)
{
    Point p;

    p.x = x + x1;
    p.y = y + y1;

    return p;
}



Point Rocket::Rotate(float x, float y)
{
    Point p;

    p.x = x*cos(theta) - y*sin(theta);
    p.y = x*sin(theta) + y*cos(theta);

    return p;
}



void Rocket::Draw()
{
	missileManager.Draw();

	if(timeLeft > 0.0f)
	{
		shootingTexture.Activate();
	}
	else
	{
		idleTexture.Activate();
	}
	
    glBegin(GL_TRIANGLE_STRIP);

	Point p = {0.0f, 0.0f};

    glTexCoord2f(0.0f, 0.0f);
    p = Rotate(-rocketSize, -rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glTexCoord2f(1.0f, 0.0f);
    p = Rotate( rocketSize, -rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);


    glTexCoord2f(0.0f, 1.0f);
    p = Rotate(-rocketSize,  rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glTexCoord2f(1.0f, 1.0f);
    p = Rotate( rocketSize,  rocketSize);
    p = Translate(p.x, p.y);
    glVertex2f(p.x, p.y);

    glEnd();
}




void Rocket::Init()
{
	idleTexture.Init("rocket.tga");
	shootingTexture.Init("rocket_firing.tga");
	missileManager.Init();
}



void Rocket::OnMouseMove(float x1, float y1)
{
	x1 -= x;
    y1 -= y;
    theta = atan2(y1, x1);
}


void Rocket::Update(float deltaTime)
{
	timeLeft -= deltaTime;
	missileManager.Update(deltaTime);
}


void Rocket::Relocate(float x1, float y1)
{
	x = x1;
	y = y1;
	theta = 0.0f;
}