# include <math.h>
# include <random>

# include "Asteroid.h"
# include "TgaTexture.h"



float GetRandomValue(float min, float max)
{
	//int number = rand();
	int number = std::rand();
	float value = float(number) / float(RAND_MAX);

	return min + value*(max-min);
}

int GetRandom(int limit)
{
	return rand() % limit;
}


Asteroid::Asteroid():back(0), next(0)
{
	Reset();
}


void Asteroid::Reset()
{
	dx = GetRandomValue(0.1f, 0.25f);
	dy = -GetRandomValue(0.25f, 0.5f);

	y = GetRandomValue(1.1f, 1.5f);
	x = GetRandomValue(0.0f, 1.0f);

	if(GetRandom(10) > 5) x = -x;
	if(GetRandom(10) > 5) dx = -dx;

	theta = GetRandomValue(-4.0f, 4.0f);

	color = GetRandom(6);
	size = GetRandomValue(0.05f, 0.1f);
}


bool Asteroid::Update(float deltaTime)
{
	theta += deltaTime*size*10.0f; 

	float speed = 0.5f;
	x += dx*speed*deltaTime;
	y += dy*speed*deltaTime;
	
	const float left = x - size;
	const float right = x + size;
	const float top = y + size;
	const float bottom = y - size;
	const float offset = 1.1f;

	if(left > offset) Reset();
	else if(right < -offset) Reset();
	else if(top < -offset) Reset();
	else if(bottom > offset) Reset();

	return false;
}


void Asteroid::Draw()
{
	float colors[6][3] =
	{
		{0.75f, 0.25f, 0.25f},
		{0.25f, 0.75f, 0.25f},
		{0.25f, 0.25f, 0.75f},
		{0.25f, 0.75f, 0.75f},
		{0.75f, 0.25f, 0.75f},
		{0.75f, 0.75f, 0.25f},
	};

	glColor3f(colors[color][0], colors[color][1], colors[color][2]);
	TgaTexture::DrawQuad(x, y, size, size, theta);
}



bool Asteroid::CheckCollision(float x1, float y1, float size1)
{
	x1 -= x;
	y1 -= y;
	float distance = sqrt(x1*x1 + y1*y1);
	
	return (distance < (size+size1));
}


bool Asteroid::Clone()
{
	size /= 2.0f;

	if(size < 0.025f) return false;

	dx = -dx;

	Asteroid* asteroid = new Asteroid();

	asteroid->x = x;
	asteroid->y = y;
	asteroid->dx = -dx;
	asteroid->dy = dy;
	asteroid->size = size;
	asteroid->color = color;

	asteroid->back = back;
	asteroid->next = this;

	if(back) back->next = asteroid;
	back = asteroid;
	return true;
}