# include <math.h>
# include <stdio.h>
# include <stdlib.h>
# include <GL/glut.h>
# include <mmsystem.h> 

# include "Rocket.h"

static const float bombTime = 0.25f;
static const float smokeTime = 0.75f;
static const float rocketSize = 0.125f;

Rocket::Rocket() : x(0.5f), y(-0.5f), theta(0.0f), timeLeft(0.0f), smokeTimeLeft(0.0f), gameOver(false)
{

}



void Rocket::Shoot()
{
	if(gameOver) return;

	if(timeLeft <= 0.0f)
	{
		timeLeft = bombTime;
	}
	
	PlaySound(L"shoot.wav", NULL, SND_ASYNC);
	missileManager.Add(x, y, theta);
}


void Rocket::DrawSmoke()
{
	if(gameOver) return;

	if(smokeTimeLeft < 0.0f) return;


	smokeTexture.Activate();

	float fi = theta + 22.0f/7.0f;

	float dx = x + rocketSize*2.0f*cos(fi);
	float dy = y + rocketSize*2.0f*sin(fi);

	float p = (smokeTime - smokeTimeLeft)/smokeTime;

	glColor4f(1.0f - p, 1.0f, 0.0f, 1.0f - p);
	TgaTexture::DrawQuad(dx, dy, rocketSize, rocketSize*0.5f, fi);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}


void Rocket::Draw()
{
	missileManager.Draw();

	if(gameOver) return;

	DrawSmoke();

	if(timeLeft > 0.0f)
		shootingTexture.Activate();
	else
		idleTexture.Activate();
	
	TgaTexture::DrawQuad(x, y, rocketSize, rocketSize, theta);
}




void Rocket::Init()
{
	idleTexture.Init("rocket.tga");
	shootingTexture.Init("rocket_firing.tga");
	smokeTexture.Init("fire.tga");
	missileManager.Init();
}



void Rocket::OnMouseMove(float x1, float y1)
{
	if(gameOver) return;

	x1 -= x;
    y1 -= y;
    theta = atan2(y1, x1);
}


void Rocket::Update(float deltaTime, AsteroidManager* asteroidManager, SpaceShip* ship)
{
	timeLeft -= deltaTime;
	smokeTimeLeft -= deltaTime;
	missileManager.Update(deltaTime, asteroidManager, ship);	
	
	if(gameOver) return;

	if(asteroidManager->CheckCollision(x, y, rocketSize))
	{
		gameOver = true;
		PlaySound(L"game_fail.wav", NULL, SND_ASYNC);
	}

	if(smokeTimeLeft <= 0.0f) return;

	float speed = 0.5f;
	x += cos(theta)*speed*deltaTime;
	y += sin(theta)*speed*deltaTime;

	if(x - rocketSize >  1.0f) 
		x -= 2.0f;
	else if(x + rocketSize < -1.0f)
		x += 2.0f;
	if(y - rocketSize >  1.0f) 
		y -= 2.0f;
	else if(y + rocketSize < -1.0f)
		y += 2.0f;
}


void Rocket::Relocate(float x1, float y1)
{
	x = x1;
	y = y1;
	theta = 0.0f;
	gameOver = false;
	timeLeft = 0.0f;
	smokeTimeLeft = 0.0f;
}


void Rocket::Progress()
{
	if(gameOver) return;

	if(smokeTimeLeft <= 0.0f)
		PlaySound(L"move.wav", NULL, SND_ASYNC);

	smokeTimeLeft = smokeTime;
}