# include "Asteroid.h"
# include "AsteroidManager.h"


AsteroidManager::AsteroidManager() : head(0)
{
}


AsteroidManager::~AsteroidManager()
{
	Destroy();
}


void AsteroidManager::Destroy()
{
	Asteroid* asteroid = head;

	while(asteroid)
	{
		Asteroid* temp = asteroid->back;
		delete asteroid;
		asteroid = temp;
	}

	head = 0;
}


void AsteroidManager::Init()
{
	asteroidImage.Init("asteroid.tga");

	srand(GetTickCount());

	for(int i = 0; i < 10; i++)
	{
		Asteroid* asteroid = new Asteroid();
		asteroid->back = head;
		if(head) head->next = asteroid;
		head = asteroid;
	}
}


void AsteroidManager::Update(float deltaTime)
{
	Asteroid* asteroid = head;

	if(deltaTime > 0.1f) deltaTime = 0.1f;

	while(asteroid)
	{
		Asteroid* temp = asteroid->back;
		asteroid->Update(deltaTime);
		asteroid = temp;
	}
}



void AsteroidManager::Draw()
{
	Asteroid* asteroid = head;

	asteroidImage.Activate();

	while(asteroid)
	{
		Asteroid* temp = asteroid->back;
		asteroid->Draw();
		asteroid = temp;
	}

	glColor3f(1.0f, 1.0f, 1.0f);
}


bool AsteroidManager::CheckCollision(float x, float y, float size, bool destroy)
{
	Asteroid* asteroid = head;
	while(asteroid)
	{
		if(asteroid->CheckCollision(x, y, size))
		{
			if(destroy || !asteroid->Clone())
				asteroid = Remove(asteroid);
			return true;
		}
		asteroid = asteroid->back;
	}

	return false;
}




Asteroid* AsteroidManager::Remove(Asteroid* asteroid)
{
	if(!asteroid) return 0;

	if(asteroid->back)
		asteroid->back->next = asteroid->next;
	if(asteroid->next)
		asteroid->next->back = asteroid->back;
	
	if(asteroid == head)
		head = asteroid->back;

	Asteroid* temp = asteroid->back;
	delete asteroid;
	return temp;
}


