# include <math.h>
# include <Windows.h>

# include "Utils.h"
# include "SpaceShip.h"
# include "TgaTexture.h"


static float spaceShipSize = 0.125f;

SpaceShip::SpaceShip() : timeLeft(1.0f)
{
	Reset();
}


void SpaceShip::Reset()
{
	if(GetRandom(2))
	{
		x = GetRandomValue(-1.0f, 1.0f);
		y = 1.0f + spaceShipSize;
		if(GetRandom(2)) y = -y;
	}
	else
	{
		y = GetRandomValue(-1.0f, 1.0f);
		x = 1.0f + spaceShipSize;
		if(GetRandom(2)) x = -x;
	}

	dx = GetRandomValue(0.5f, 1.0f);
	dy = GetRandomValue(0.5f, 1.0f);

	if(x > 0.0f) dx = -dx;
	if(y > 0.0f) dy = -dy;

	color = GetRandom(3);

	gun.Reset();
	timeLeft = 0.25f;
}


void SpaceShip::Update(float deltaTime, float x1, float y1)
{
	float speed = 0.5f;
	x += dx*speed*deltaTime;
	y += dy*speed*deltaTime;
	
	const float left = x - spaceShipSize;
	const float right = x + spaceShipSize;
	const float top = y + spaceShipSize;
	const float bottom = y - spaceShipSize;
	const float offset = 1.1f;

	if(left > offset) Reset();
	else if(right < -offset) Reset();
	else if(top < -offset) Reset();
	else if(bottom > offset) Reset();

	smoke.Update(deltaTime);

	gun.Update(deltaTime);

	timeLeft -= deltaTime;
	if(timeLeft < 0.0f)
	{
		gun.Add(x, y, x1, y1);
		timeLeft = 0.25f;
	}
}


void SpaceShip::Draw()
{
	SetColor(color);
	TgaTexture::DrawQuad(x, y, spaceShipSize, spaceShipSize, 0.0f);
	smoke.Draw();
	gun.Draw();
}



bool SpaceShip::CheckCollision(float x1, float y1, float size)
{	
	x1 -= x;
	y1 -= y;
	float distance = sqrt(x1*x1 + y1*y1);
	
	if(distance < (size+spaceShipSize))
	{
		PlaySound(L"blast.wav", NULL, SND_ASYNC);
		smoke.Activate(x, y, color);
		Reset();
		return true;
	}

	return false;
}


void SpaceShip::Init()
{
	smoke.Init();
	gun.Init();
}
